#ifndef LANDED_MOVIE
#define LANDED_MOVIE

#include <QThread>
#include <QString>
#include <QProcess>

#include "viewer3d.hh"

class Movie : public QThread
{
Q_OBJECT
public:
  Movie(Viewer3D*);

 private:
  //pipe to ffmpeg for movie recording
  Viewer3D* src_;
  QProcess pipe_;
  bool run_;

 public:
  bool setup(QString);
  void stop(void);

 private:
  void run(void);

};

#endif //ifndef LANDED_MOVIE
