EOF

chmod +x $installdir/landed
which landed &>/dev/null || echo "$installdir does not seem to be in $PATH"

missing=`ldd $installdir/landed | grep 'not found' | cut -d= -f 1`
if [ ! -z "$missing" ]; then
    echo "unsatisfied dependencies:"
    for file in $missing
    do
	echo -e "\t$file"
    done
fi
