import re

file=open('pdgmap.icc', 'r')

for line in file:
    m=re.search('(.*,\s*QString)(\(.*)', line)
    if m:
        print m.group(1)+"::fromUtf8"+m.group(2)
    else:
        print line
file.close()
