#ifndef QT_LED
#define QT_LED

#include <QWidget>

class LED : public QWidget
{
 Q_OBJECT
 public:
  LED(QWidget*, int=24);

 private:
  bool on_;
  int size_;

 private:
  void paintEvent(QPaintEvent*);

 public:
  void setOn(void);
  void setOff(void);
  void set(bool);
  bool on(void) const;

};

#endif //ifndef QT_LED
