#ifndef LANDED_APP
#define LANDED_APP

#include <QObject>
#include <QApplication>
#include <QEvent>

#include "gui.hh"

class Landed : public QApplication
{
Q_OBJECT

public:
  Landed(int, char*[]);

 private:
  Gui gui_;

 protected:
  bool event(QEvent*);

};

#endif //ifndef LANDED_APP
