#!/bin/sh

echo "Enter location for landed app (default=$HOME/bin)"
read installdir
[ -z $installdir ] && installdir=$HOME/bin
if [ ! -d $installdir ]; then
    echo "creating directory $installdir"
    mkdir -p $installdir || (echo "Failed to create directory $installdir";exit 1)
fi
echo "installing"
base64 -d > $installdir/landed << EOF
